/* ==========================================================================
 * Name:        tlenek\image.h
 * Purpose:     Deklaracja klasy Image reprezentujacej pojedynczy obrazek
 *              wykorzystywana w komunikatorze i bibliotece
 *              oraz prosta nakladka obiektowa ulatwiajaca operowanie na 
 *              obiekcie bitmapy (HBITMAP)
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     21/09/2007 02:01:56
 * SVN-ID:      $Id: image.h 25 2007-11-22 16:43:37Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_IMAGE_H_
#define _TLENEK_IMAGE_H_

#include <windows.h>
#include "tlenek\defs.h"
#include "tlenek\misc.h"
#include "tlenek\utils.h"
#include "tlen_sdk\plugin_skin.h"


namespace tlenek {


// prosta obiektowka na HBITMAP
class TLENEK_DLLIMPEXP Bitmap {

	private:

		HBITMAP m_bmp;
		long m_width;
		long m_height;

		Bitmap(const Bitmap&) {}


	public:

		Bitmap() {
			m_bmp = NULL;
		}

		Bitmap(int resource) {
			Load(resource);
		}

		Bitmap(const char* filename) {
			Load(filename);
		}

		Bitmap(HBITMAP hbmp) {
			Set(hbmp);
		}
		
		~Bitmap() {
			DeleteObject(m_bmp);
		}


		void Load(int resource) {
			DeleteObject(m_bmp);
			m_bmp = (HBITMAP) LoadImage(GetPluginInstance(), MAKEINTRESOURCE(resource), IMAGE_BITMAP, 0, 0, NULL);
			UpdateSize();
		}

		void Load(const char* filename) {
			DeleteObject(m_bmp);
			m_bmp = (HBITMAP) LoadImage(NULL, filename, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			UpdateSize();
		}

		const HBITMAP Get() const {
			return m_bmp;
		}

		void Set(HBITMAP hbmp) {
			m_bmp = (HBITMAP) CopyImage(hbmp, IMAGE_BITMAP, 0, 0, 0);
			UpdateSize();
		}
		
		long GetWidth() const {
			return m_width;
		}

		long GetHeight() const {
			return m_height;
		}


	private:

		void UpdateSize() {
			BITMAP bmp;
			GetObject(m_bmp, sizeof(BITMAP), &bmp);
			m_width = bmp.bmWidth;
			m_height = bmp.bmHeight;
		}

};



class TLENEK_DLLIMPEXP Image : private TlenSingleImageDef {

	private:

		void Init() {
			structSize = sizeof(TlenSingleImageDef);
			SkinID = NULL;
			Bitmap = 0;
			Transparent = 0;
			::SetRect(&Location, 0, 0, 0, 0);
		}


	public:

		Image() {
			Init();
		}

		Image(const char* id) {
			Init();
			SetSkinId(id);
		}


		Image(const HBITMAP& bmp, const COLORREF& color = 0xFF00FF) {
			structSize = sizeof(TlenSingleImageDef);
			SkinID = NULL;
			SetBitmap(bmp);
			SetTransparentColor(color);
			SetDefaultRect();
		}

		Image(const class Bitmap* bmp, const COLORREF& color = 0xFF00FF) {
			structSize = sizeof(TlenSingleImageDef);
			SkinID = NULL;
			SetBitmap(bmp->Get());
			SetTransparentColor(color);
			::SetRect(&Location, 0, 0, bmp->GetWidth(), bmp->GetHeight());
		}

		Image(const Image& image) {
	//		CopyTlenSingleImageDef(this, &image);
			structSize = image.structSize;
			my_cstrdup(SkinID, image.SkinID, false);
			Bitmap = image.Bitmap;
			Transparent = image.Transparent;
			CopyRect(&Location, &image.Location);
		}

		~Image() {
			free(SkinID);
		}


		const char* GetSkinId() const {
			return SkinID;
		}

		void SetSkinId(const char* id) {
			my_cstrdup(SkinID, id);
		}

		COLORREF GetTransparentColor() const {
			return Transparent;
		}

		void SetTransparentColor(const COLORREF& color) {
			Transparent = color;
		}

		// polozenie grafiki na bitmapie
		// nie jest const, bo mozna zmienic bezposrednio wartosci
		RECT* GetRect() {
			return &Location;
		}

		void SetRect(const RECT* rect) {
			CopyRect(&Location, rect);
		}

		// defualtowa lokalizacja, czyli 0,0,width,height
		void SetDefaultRect() {
			BITMAP bmp;
			GetObject(Bitmap, sizeof(BITMAP), &bmp);
			::SetRect(&Location, 0, 0, bmp.bmWidth, bmp.bmHeight);
		}


		const HBITMAP GetBitmap() const {
			return Bitmap;
		}

		void SetBitmap(const HBITMAP hbmp) {
			Bitmap = hbmp;
		}

		void SetBitmap(const class Bitmap* bmp) {
			Bitmap = bmp->Get();
		}


		bool operator==(const Image& img) {
			return (strcmp(SkinID, img.SkinID) == 0 && Bitmap == img.Bitmap 
					&& Transparent == img.Transparent 
					&& memcmp(&Location, &img.Location, sizeof(RECT)) == 0);
		}


	friend class ContactIcon;

};


inline void CopyTlenSingleImageDef(TlenSingleImageDef* dst, const TlenSingleImageDef* src) {
	dst->structSize = src->structSize;
	my_cstrdup(dst->SkinID, src->SkinID, false);
	dst->Bitmap = src->Bitmap;
	dst->Transparent = src->Transparent;
	CopyRect(&dst->Location, &src->Location);
}

inline void FreeTlenSingleImageDef(TlenSingleImageDef* dst) {
	free(dst->SkinID);
}


} // namespace tlenek


#endif // _TLENEK_IMAGE_H_
