/* ==========================================================================
 * Name:        tlenek\xml2.h
 * Purpose:     Deklaracja klas parsera XML wykorzystujacego libexpat2.dll
 * Author:      Marcin 'malcom' Malich <me@malcom.pl>
 * Modified by:
 * Created:     14/11/2007 12:31:02
 * SVN-ID:      $Id: xml2.h 24 2007-11-20 14:23:04Z malcom $
 * Copyright:   (c) 2007 Marcin 'malcom' Malich <me@malcom.pl>, Poland
 * Licence:     GNU Lesser General Public License
 * ========================================================================== */

#ifndef _TLENEK_XML2_H_
#define _TLENEK_XML2_H_

#include "tlenek\defs.h"
#include "tlenek\info.h"


namespace tlenek {
namespace xml2 {


#pragma pack (1)
#pragma pack (push)


/* pheap - singular allocation of memory */
struct pheap {
	void* block;
	int size, used;
};

/* pool_cleaner - callback type which is associated with a
   pool entry; invoked when the pool entry is free'd */
typedef void (*pool_cleaner)(void* arg);

/* pfree - a linked list node which stores an
   allocation chunk, plus a callback */
struct pfree {
	pool_cleaner f;
	void* arg;
	struct pheap* heap;
	struct pfree* next;
};

/* pool - base node for a pool. Maintains
   a linked listof pool entries (pfree) */
typedef struct pool_struct {
	int size;
	struct pfree* cleanup;
	struct pheap* heap;
} _pool, *pool;


/* xmlnodes - Document Object Model */

#define NTYPE_TAG		0	/* xmlnode is an element (tag) */
#define NTYPE_ATTRIB	1	/* xmlnode is an attribute node */
#define NTYPE_CDATA		2	/* xmlnode is a text node (!) */

#define NTYPE_LAST		2	/* highest possible value of xmlnode types */
#define NTYPE_UNDEF	   -1	/* xmlnode has no defined type */


typedef struct xmlnode_t {
	char*				name;		/* name of the xmlnode */
	unsigned short		type;		/* type of the xmlnode, one of ::NTYPE_TAG, ::NTYPE_ATTRIB, ::NTYPE_CDATA, or ::NTYPE_UNDEF */
	char*				data;		/* data of the xmlnode, for attributes this is the value, for text nodes this is the text */
	int					data_sz;	/* length of the data in the xmlnode */
	int					complete;
	pool				p;			/* memory pool used by this xmlnode (the same as for all other xmlnode in a tree) */
	struct xmlnode_t*	parent;		/* parent node for this node, or NULL for the root element */
	struct xmlnode_t*	firstchild;	/* first child element of this node, or NULL for no child elements */
	struct xmlnode_t*	lastchild;	/* last child element of this node, or NULL for no child elements */
	struct xmlnode_t*	prev;		/* previous sibling */
	struct xmlnode_t*	next;		/* next sibling */
	struct xmlnode_t*	firstattrib;/* first attribute node of this node */
	struct xmlnode_t*	lastattrib;	/* last attribute node of this node */
} _xmlnode, *xmlnode;



/* XSTREAM */

#define XSTREAM_MAXNODE		1000000
#define XSTREAM_MAXDEPTH	100

#define XSTREAM_ROOT		0	/* root element */
#define XSTREAM_NODE		1	/* normal node */
#define XSTREAM_CLOSE		2	/* closed </stream:stream> */
#define XSTREAM_ERR			4	/* parser error */

typedef void (*xstream_onNode)(int type, xmlnode x, void* arg);	/* xstream event handler */

typedef struct xstream_struct {
//	XML_Parser parser;
	xmlnode node;
	char* cdata;
	int cdata_len;
	pool p;
	xstream_onNode f;
	void* arg;
	int status;
	int depth;
} *xstream, _xstream;


#pragma pack (pop)


/* funkcje eksportowane w libexpat2.dll */

typedef pool    (*_pool_new_t)(char* zone, int line);
typedef int     (*j_atoi_t)(const char* a, int def);
typedef char*   (*j_strcat_t)(char* dest, char* txt);
typedef int     (*j_strcmp_t)(const char* a, const char* b);
typedef char*   (*j_strdup_t)(const char *str);
typedef int     (*j_strncasecmp_t)(const char* a, const char* b, int i);
typedef int     (*j_strncmp_t)(const char* a, const char* b, int i);
typedef void    (*pool_free_t)(pool p);
typedef int     (*xmlnode2file_t)(char* file, xmlnode node);
typedef char*   (*xmlnode2str_t)(xmlnode node);
typedef xmlnode (*xmlnode_dup_t)(xmlnode x);
typedef xmlnode (*xmlnode_file_t)(char* file);
typedef void    (*xmlnode_free_t)(xmlnode node);
typedef char*   (*xmlnode_get_attrib_t)(xmlnode owner, const char* name);
typedef char*   (*xmlnode_get_data_t)(xmlnode node);
typedef xmlnode (*xmlnode_get_firstchild_t)(xmlnode parent);
typedef char*   (*xmlnode_get_name_t)(xmlnode node);
typedef xmlnode (*xmlnode_get_nextsibling_t)(xmlnode sibling);
typedef xmlnode (*xmlnode_get_tag_t)(xmlnode parent, const char* name);
typedef char*   (*xmlnode_get_tag_data_t)(xmlnode parent, const char* name);
typedef xmlnode (*xmlnode_insert_cdata_t)(xmlnode parent, const char* CDATA, unsigned int size);
typedef xmlnode (*xmlnode_insert_tag_t)(xmlnode parent, const char* name);
typedef xmlnode (*xmlnode_new_tag_t)(const char* name);
typedef void    (*xmlnode_put_attrib_t)(xmlnode owner, const char* name, const char* value);
typedef xmlnode (*xmlnode_str_t)(char* str, int len);
typedef xstream (*xstream_new_t)(pool p, xstream_onNode f, void* arg);
typedef int     (*xstream_eat_t)(xstream xs, char* buff, int len);


extern xmlnode2file_t xmlnode2file;
extern xmlnode2str_t xmlnode2str;
extern xmlnode_file_t xmlnode_file;
extern xmlnode_free_t xmlnode_free;
extern xmlnode_get_attrib_t xmlnode_get_attrib;
extern xmlnode_get_data_t xmlnode_get_data;
extern xmlnode_get_tag_t xmlnode_get_tag;
extern xmlnode_get_tag_data_t xmlnode_get_tag_data;
extern xmlnode_str_t xmlnode_str;


static HMODULE hModule = NULL;

TLENEK_DLLIMPEXP void Init(const char* dir);

inline void Deinit() {
	FreeLibrary(hModule);
}


class TLENEK_DLLIMPEXP Attribute : private xmlnode_t {

	private:

		Attribute() {}
		~Attribute() {}


	public:

		const char* GetName() const {
			return name;
		}

		const char* GetValue() const {
			return data;
		}

		Attribute* GetPrev() const {
			return (Attribute*) prev;
		}

		Attribute* GetNext() const {
			return (Attribute*) next;
		}

};


class TLENEK_DLLIMPEXP Node : private xmlnode_t {

	private:

		Node() {}
		~Node() {}


	public:

		enum NodeType {
			TYPE_UNDEF = NTYPE_UNDEF,
			TYPE_TAG = NTYPE_TAG,
			TYPE_ATTRIB = NTYPE_ATTRIB,
			TYPE_CDATA = NTYPE_CDATA
		};

		NodeType GetType() const {
			return (NodeType) type;
		}


		const char* GetName() const {
			return name;
		}

		const char* GetContent() const {
			return xmlnode_get_data((xmlnode_t*)this);
		}


		Node* GetParent() const {
			return (Node*) parent;
		}

		Node* GetChildren() const {
			return (Node*) firstchild;
		}


		Node* GetPrev() const {
			return (Node*) prev;
		}

		Node* GetNext() const {
			return (Node*) next;
		}


		Node* GetTag(const char* name) {
			return (Node*) xmlnode_get_tag(this, name);
		}

		const char* GetTagData(const char* name) {
			return xmlnode_get_tag_data(this, name);
		}


		bool HasChild() const {
			return firstchild != NULL;
		}

		bool HasAttrib() {
			return firstattrib != NULL;
		}


		const char* GetAttribute(const char* name) const {
			return xmlnode_get_attrib((xmlnode_t*)this, name);
		}

		Attribute* GetAttributes() const {
			return (Attribute*) firstattrib;
		}


};


class TLENEK_DLLIMPEXP Document {

	private:

		xmlnode m_root;

		void Init() {
			// w przypadku budowania pluginu na tlenku
			// parser XML automatycznie inicjujemy w razie potrzeby,
			// zwalniany jest w destruktorze pluginu
			if (Plugin::GetInstance() != NULL && hModule == NULL) {
				xml2::Init(info::GetTlenPath());
			}
		}


	public:

		Document() {
			Init();
			m_root = NULL;
		}

		Document(const char* filename) {
			Init();
			Load(filename);
		}

		~Document() {
			xmlnode_free(m_root);
		}


		bool Load(const char* filename) {
			m_root = xmlnode_file((char*) filename);
			return IsOk();
		}

		bool Save(const char* filename) {
			return xmlnode2file((char*) filename, m_root) == 1;
		}


		bool FromString(const char* str, size_t len) {
			m_root = xmlnode_str((char*) str, len);
			return IsOk();
		}

		bool ToString(char* buf, size_t* len);


		bool IsOk() const {
			return m_root != NULL;
		}


		Node* GetRoot() const {
			return (Node*) m_root;
		}

};


} // namespace xml2
} // namespace tlenek


#endif // _TLENEK_XML2_H_
